/* --COPYRIGHT--,BSD
* Copyright (c) 2014, Texas Instruments Incorporated
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
* * Redistributions of source code must retain the above copyright
* notice, this list of conditions and the following disclaimer.
*
* * Redistributions in binary form must reproduce the above copyright
* notice, this list of conditions and the following disclaimer in the
* documentation and/or other materials provided with the distribution.
*
* * Neither the name of Texas Instruments Incorporated nor the names of
* its contributors may be used to endorse or promote products derived
* from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
* OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
* --/COPYRIGHT--*/

#include <intrinsics.h>
#include <stdint.h>
#include "USB_API/USB_Common/types.h"               //Basic Type declarations
#include "TI_MSP430.h"
#include "TI_MSP430_hardware_board.h"
#include "host_interface.h"
#include "timer0.h"
#include "timer2.h"
#include "lmt01_1gpio_transistor_modes.h"
#include "lmt01_1gpio_comparator_modes.h"
#include "lmt01_1timer_transistor_modes.h"
#include "lmt01_1timer_comparator_modes.h"
#include "lmt01_2timer_transistor_modes.h"
#include "lmt01_2timer_comparator_modes.h"

extern uint8_t single_shot_state;
extern uint8_t cont_graph_state;
extern uint16_t timer_trigger_freq;
extern uint8_t bufferX[], bufferY[];
extern uint8_t *mbuf, *ubuf;
extern uint8_t xy_buf_length;
extern uint8_t save_xy_buf_length;
extern uint8_t USB_disable(void);
extern uint8_t *ubuf;
extern volatile uint8_t LMT01_XYBufferReady;
extern uint8_t lmt01_measure_mode;
extern volatile uint16_t isr_pulse_count;
extern volatile uint8_t isr_pulse_detect;

void handle_reInit(uint8_t *pBuf, uint16_t pSize);

uint8_t char2nibble(uint8_t db);      
       

// Parse Command, then execute
uint8_t handleHostCommand(uint8_t *pBuf, uint16_t pSize)
{
  uint8_t host_response = 0;
  uint16_t word_data;
  uint8_t nxt_byte, byte_data;
  
  nxt_byte = char2nibble(pBuf[0]);
  nxt_byte = (nxt_byte << 4) + char2nibble(pBuf[1]);
  switch(nxt_byte)
  {
    case Command_LoopPacket:  
    {
      host_response = 1;
      break;
    }
    case Command_ReInit: 
    {
      handle_reInit(pBuf, pSize);
      host_response = 1;
      break;
    }    
    case Command_Read_Samples_One_Shot:
    {
      switch (lmt01_measure_mode) 
      {
         case SINGLE_GPIO_SYNC_TRANSISTOR_MODE:
           {
              lmt01_oneshot_1gpio_sync_transistor_mode(pBuf); 
              break;
           }
         case SINGLE_GPIO_ASYNC_TRANSISTOR_MODE:
           {
              lmt01_oneshot_1gpio_async_transistor_mode(pBuf); 
              break;
           }
         case SINGLE_GPIO_SYNC_COMPARATOR_MODE:
           {
              lmt01_oneshot_1gpio_sync_comparator_mode(pBuf); 
              break;
           }
         case SINGLE_GPIO_ASYNC_COMPARATOR_MODE:
           {
              lmt01_oneshot_1gpio_async_comparator_mode(pBuf); 
              break;
           }           
         case SINGLE_TIMER_SYNC_TRANSISTOR_MODE:
           {
              lmt01_oneshot_1timer_sync_transistor_mode(pBuf);
              break;
           }
         case SINGLE_TIMER_ASYNC_TRANSISTOR_MODE:
           {
              lmt01_oneshot_1timer_async_transistor_mode(pBuf); 
              break;
           } 
         case SINGLE_TIMER_SYNC_COMPARATOR_MODE:
           {
              lmt01_oneshot_1timer_sync_comparator_mode(pBuf);
              break;
           }
         case SINGLE_TIMER_ASYNC_COMPARATOR_MODE:
           {
              lmt01_oneshot_1timer_async_comparator_mode(pBuf); 
              break;
           }            
         case TWO_TIMER_SYNC_TRANSISTOR_MODE:
           {
              lmt01_oneshot_2timer_sync_transistor_mode(pBuf);
              break;
           }
         case TWO_TIMER_ASYNC_TRANSISTOR_MODE:
           {
              lmt01_oneshot_2timer_async_transistor_mode(pBuf);
              break;
           }           
         case TWO_TIMER_SYNC_COMPARATOR_MODE:
           {             
              lmt01_oneshot_2timer_sync_comparator_mode(pBuf);
              break;
           }            
         case TWO_TIMER_ASYNC_COMPARATOR_MODE:
           {
              lmt01_oneshot_2timer_async_comparator_mode(pBuf);
              break;
           }           
         default:
           {
              break;
           }
      }            
      host_response = 1;
      break;
    }
    case Command_Start_Graph:
    {
      uint16_t graph_period;
      
      // low byte first
      graph_period = char2nibble(pBuf[4]);
      graph_period = (graph_period <<4) + char2nibble(pBuf[5]);
      // high byte next
      graph_period = (graph_period <<4) + char2nibble(pBuf[2]);
      graph_period = (graph_period <<4) + char2nibble(pBuf[3]);
      
      save_xy_buf_length = xy_buf_length;
      xy_buf_length= 2;
      LMT01_XYBufferReady = 0;           

      if ((lmt01_measure_mode == TWO_TIMER_SYNC_TRANSISTOR_MODE) ||
          (lmt01_measure_mode == TWO_TIMER_SYNC_COMPARATOR_MODE) ||
          (lmt01_measure_mode == TWO_TIMER_ASYNC_TRANSISTOR_MODE) ||
          (lmt01_measure_mode == TWO_TIMER_ASYNC_COMPARATOR_MODE))
      {
        // only 100ms graph period supported by stop watch timer
        if (graph_period == GRAPH_100MS_PERIOD)
        {
          cont_graph_state = 1;
          lmt01_restart_timers();
        } else
        {
          // only 100ms graph period supported by stop watch timer, for any
          // other period repeatedly send read_sample_one_shot command to get data
          cont_graph_state = 0;
        }
          
      } else 
        // SINGLE_TIMER SYNC/ASYNC Mode, SINGLE GPIO SYNC/ASYNC Mode
      {
        // no 50ms stop watch timer to continuously get data
        // repeatedly send read_sample_one_shot command to get data
        cont_graph_state = 0;
      }        
      host_response = 1;
      break;
    }
    case Command_End_Graph:
    {  
      cont_graph_state = 0;      
      if ((lmt01_measure_mode == TWO_TIMER_SYNC_TRANSISTOR_MODE) ||
          (lmt01_measure_mode == TWO_TIMER_SYNC_COMPARATOR_MODE) ||
          (lmt01_measure_mode == TWO_TIMER_ASYNC_TRANSISTOR_MODE) ||
          (lmt01_measure_mode == TWO_TIMER_ASYNC_COMPARATOR_MODE))
      {
        // stop Pulse Count Timer and Stop Watch Timer
        lmt01_stop_timers();        
      }
      TI_LMT01_XFER_LED_PxOUT |= TI_LMT01_XFER_LED_PIN;                // Set XFER_LED
      TI_LMT01_TEST_LED_PxOUT |= TI_LMT01_TEST_LED_PIN;                // Set TEST_LED      
      xy_buf_length = save_xy_buf_length;
      host_response = 1;
      break;
    }
  case Command_Firmware_Version_Read:
    {
      pBuf[8] = Firmware_VersionA;
      pBuf[9] = Firmware_VersionB;
      pBuf[10] = Firmware_VersionC;
      pBuf[11] = Firmware_VersionD;
      host_response = 1;
      break;
    }
  case Command_LED_Toggle:
    {
      // toggle TEST_LED
      TI_LMT01_TEST_LED_PxOUT ^= TI_LMT01_TEST_LED_PIN;
      host_response = 1;
      break;
    }
  case Command_Read_Num_Samp_Oneshot:
    {
      // num of samples = buf_length/2
      pBuf[8] = xy_buf_length>>1;
      host_response = 1;
      break;
    }
  case Command_Write_Num_Samp_Oneshot:
    {
      byte_data = char2nibble(pBuf[2]);
      byte_data = (byte_data <<4) + char2nibble(pBuf[3]);
      // buf length = num of samples * 2
      xy_buf_length= byte_data<<1;
      host_response = 1;
      break;
    }
  case Command_Read_Comparator_Low_Threshold:
    {
      pBuf[8] = CBCTL2 & 0x001F;
      host_response = 1;
      break;
    }
  case Command_Write_Comparator_Low_Threshold:
    {
      byte_data = char2nibble(pBuf[2]);
      byte_data = (byte_data <<4) + char2nibble(pBuf[3]);
      CBCTL2 &= 0xFFE0;
      CBCTL2 |= byte_data;
      host_response = 1;
      break;
    }    
  case Command_Read_Comparator_High_Threshold:
    {
      pBuf[8] = (CBCTL2 >> 8) & 0x001F;
      host_response = 1;
      break;
    }
  case Command_Write_Comparator_High_Threshold:
    {
      byte_data = char2nibble(pBuf[2]);
      byte_data = (byte_data <<4) + char2nibble(pBuf[3]);
      CBCTL2 &= 0xE0FF;
      CBCTL2 |= (uint16_t) (byte_data << 8);
      host_response = 1;
      break;
    }
  case Command_Read_Comparator_Reference_Level:
    {
      pBuf[8] = (CBCTL2 >> 13) & 0x0003;
      host_response = 1;
      break;
    }
  case Command_Write_Comparator_Reference_Level:
    {
      byte_data = char2nibble(pBuf[2]);
      byte_data = (byte_data <<4) + char2nibble(pBuf[3]);
      CBCTL2 &= 0x9FFF;
      CBCTL2 |= (uint16_t) (byte_data << 13);
      host_response = 1;
      break;
    } 
  case Command_Read_Comparator_Filter_Delay:
    {
      pBuf[8] = (CBCTL1 >> 6) & 0x0003;
      host_response = 1;
      break;
    } 
  case Command_Write_Comparator_Filter_Delay:
    {
      byte_data = char2nibble(pBuf[2]);
      byte_data = (byte_data <<4) + char2nibble(pBuf[3]);
      CBCTL1 &= 0xFF3B;
      // valid filter delay setting is 0 to 3, 4 implies disable o/p filter
      if (byte_data < 0x04)
      {
        // enable output filter before changing delay
        CBCTL1 |= 0x0004;       
        CBCTL1 |= (uint16_t) ((byte_data & 0x03) << 6);  
      } else
      {
        // leave o/p filter disabled and delay set to 0
        CBCTL1 &= 0xFF3B;
      }
      host_response = 1;      
      break;
    } 
  case Command_Read_Comparator_Minus_Input_Select:
    {
      pBuf[8] = (CBCTL0 >> 8) & 0x000F;
      host_response = 1;
      break;
    }
  case Command_Write_Comparator_Minus_Input_Select:
    {
      byte_data = char2nibble(pBuf[2]);
      byte_data = (byte_data <<4) + char2nibble(pBuf[3]);
      CBCTL0 &= 0x70FF;
      // valid channel selection is 0 to F, disable minus > 0xF
      if (byte_data < 0x10)
      { 
        // enable comparator minus input before selecting channel
        CBCTL0 |= 0x8000;       
        CBCTL0 |= (uint16_t) (byte_data << 8);  
        // Input Buffer Disable @CBx
        CBCTL3 |= byte_data;
      } else
      {
        // leave comparator minus input disabled & channel set to 0
        CBCTL0 &= 0x70FF;
      }
      host_response = 1;      
      break;
    }
  case Command_Read_Comparator_Power_Mode:
    {
      pBuf[8] = (CBCTL1 >> 8) & 0x0003;
      host_response = 1;
      break;
    }
  case Command_Write_Comparator_Power_Mode:
    {
      byte_data = char2nibble(pBuf[2]);
      byte_data = (byte_data <<4) + char2nibble(pBuf[3]);
      CBCTL1 &= 0xFCFF;
      // set power mode
      CBCTL1 |= (uint16_t) ((byte_data & 0x03) << 8);  
      host_response = 1;      
      break;
    }
  case Command_Read_Comparator_Control_Register:
    {
      byte_data = char2nibble(pBuf[2]);
      byte_data = (byte_data <<4) + char2nibble(pBuf[3]);
      switch (byte_data) {
      case 0:
        pBuf[8] = CBCTL0;
        pBuf[9] = CBCTL0 >> 8;
        break;
      case 1:
        pBuf[8] = CBCTL1;
        pBuf[9] = CBCTL1 >> 8;
        break;
      case 2:
        pBuf[8] = CBCTL2;
        pBuf[9] = CBCTL2 >> 8;
        break;
      case 3:
        pBuf[8] = CBCTL3;
        pBuf[9] = CBCTL3 >> 8;
        break;        
      default:
        break;
      }
      host_response = 1;
      break;
    }
  case Command_Write_Comparator_Control_Register:
    {
      byte_data = char2nibble(pBuf[2]);
      byte_data = (byte_data <<4) + char2nibble(pBuf[3]);
      // high byte first
      word_data = char2nibble(pBuf[6]);
      word_data = (word_data <<4) + char2nibble(pBuf[7]);
      // low byte next
      word_data = (word_data <<4) + char2nibble(pBuf[4]);
      word_data = (word_data <<4) + char2nibble(pBuf[5]);
      switch (byte_data) {
      case 0:
        CBCTL0 = word_data;
        break; 
      case 1:
        CBCTL1 = word_data;
        break;
      case 2:
        CBCTL2 = word_data;
        break;
      case 3:
        CBCTL3 = word_data;
        break;        
      default:
        break;
      }
      host_response = 1;
      break;
    }
   case Command_Set_Timer_Trigger_Freq:
    {                              
      // low byte first
      word_data = char2nibble(pBuf[4]);
      word_data = (word_data <<4) + char2nibble(pBuf[5]);
      // high byte next
      word_data = (word_data <<4) + char2nibble(pBuf[2]);
      word_data = (word_data <<4) + char2nibble(pBuf[3]); 

      timer_trigger_freq = word_data;
      
      host_response = 1;
      break;
    }
   case Command_Read_Timer_Trigger_Freq:
    {
      pBuf[8] = timer_trigger_freq;
      pBuf[9] = timer_trigger_freq >> 8;
      host_response = 1;
      break;
    }
  case Command_Enter_MSP430_BSL:
    {
      USB_disable();
      __disable_interrupt();
      ((void (*)())0x1000)();
      host_response = 1;
      break;
    }
  case Command_Set_LMT01_Measure_Mode:
    {
      byte_data = char2nibble(pBuf[2]);
      byte_data = (byte_data <<4) + char2nibble(pBuf[3]);
      lmt01_measure_mode = byte_data;
      switch (lmt01_measure_mode) 
      {
         case SINGLE_GPIO_SYNC_TRANSISTOR_MODE:
         case SINGLE_GPIO_SYNC_COMPARATOR_MODE:
           {
              // make sure power is off for sync method
              lmt01_remove_power();             
              // Init 1GPIO for pulse count 
              init_1gpio_mode_input();
              break;
           }
         case SINGLE_GPIO_ASYNC_TRANSISTOR_MODE:
         case SINGLE_GPIO_ASYNC_COMPARATOR_MODE:
           {
              // start afresh: remove & apply power
              lmt01_remove_power();              
              // apply power
              lmt01_apply_power();
              // Init 1GPIO for pulse count 
              init_1gpio_mode_input();
              break;
           }                      
         case SINGLE_TIMER_SYNC_TRANSISTOR_MODE:
         case SINGLE_TIMER_SYNC_COMPARATOR_MODE:  
           {
              // make sure power is off for sync method
              lmt01_remove_power();             
              // Init 1timer input for pulse count 
              init_1timer_mode_input();
              break;
           }
         case SINGLE_TIMER_ASYNC_TRANSISTOR_MODE:
         case SINGLE_TIMER_ASYNC_COMPARATOR_MODE:  
           {
              // start afresh: remove & apply power
              lmt01_remove_power();              
              // apply power
              lmt01_apply_power();
              break;
           }           
         case TWO_TIMER_SYNC_TRANSISTOR_MODE:
         case TWO_TIMER_SYNC_COMPARATOR_MODE:
           {
              // make sure power is off for sync method
              lmt01_remove_power();             
              // Init 2timer input for pulse count                           
              init_2timer_mode_input();
              break;
           }
         case TWO_TIMER_ASYNC_TRANSISTOR_MODE:
         case TWO_TIMER_ASYNC_COMPARATOR_MODE:
           {
              // start afresh: remove & apply power
              lmt01_remove_power();              
              // apply power
              lmt01_apply_power();             
              break;
           }           
         default:
           {
              break;
           }
      } 
      // Reinitialize timer0 and timer2 for the new measure mode
      timer0_A0_init();
      timer2_A0_init();
      host_response = 1;
      
    }
  case Command_Read_LMT01_Measure_Mode:
    {
      pBuf[8] = lmt01_measure_mode;
      host_response = 1;
      break;
    }    
  default:
    {
      host_response = 1;
      break;
    }    
  }
  return (host_response);
}

void handle_reInit(uint8_t *pBuf, uint16_t pSize)
{
}
      
uint8_t char2nibble(uint8_t db)
{
  if ((db >= '0') && (db <= '9'))
    return (db-'0');
  else if ((db >= 'A') && (db <= 'F'))
    return (db-'A'+0xa);
  else if ((db >= 'a') && (db <= 'f'))
    return (db-'a'+0xa);
  else
    return (db);
}
