/* --COPYRIGHT--,BSD
* Copyright (c) 2014, Texas Instruments Incorporated
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
* * Redistributions of source code must retain the above copyright
* notice, this list of conditions and the following disclaimer.
*
* * Redistributions in binary form must reproduce the above copyright
* notice, this list of conditions and the following disclaimer in the
* documentation and/or other materials provided with the distribution.
*
* * Neither the name of Texas Instruments Incorporated nor the names of
* its contributors may be used to endorse or promote products derived
* from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
* OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
* --/COPYRIGHT--*/
//******************************************************************************
#include <intrinsics.h>
#include <stdint.h>
#include "USB_API/USB_Common/types.h"               //Basic Type declarations
#include "TI_MSP430.h"
#include "TI_MSP430_hardware_board.h"
#include "host_interface.h"
#include "timer0.h"
#include "timer2.h"
#include "lmt01_2timer_transistor_modes.h"
#include "lmt01_1timer_transistor_modes.h"
#include "lmt01_1gpio_transistor_modes.h"

extern uint8_t single_shot_state;
extern volatile uint16_t isr_pulse_count;
extern volatile uint8_t isr_pulse_detect;
extern uint8_t lmt01_measure_mode;

//******************************************************************************
void lmt01_oneshot_1timer_sync_transistor_mode(uint8_t *pBuf)
{
  uint16_t i;
  uint16_t pcount = 0;  
  
  single_shot_state = 1;
  
  // start afresh: remove & apply power
  lmt01_remove_power();   
  // apply power
  lmt01_apply_power();
  
  // clear & restart (previously stoppped) pulse count timer
  TA0CTL |= TACLR;
  TA0CTL |= MC_2;
  // wait 101ms
  for(i=0; i<=100; i++)
     __delay_cycles(4000);  // 1ms at 4MHz
  // stop pulse count timer
  TA0CTL &= ~MC_2;
  // save pulse count and clear timer counter 
  pcount = TA0R;
  TA0CTL |= TACLR;         
  // stop pulse count timer
  TA0CTL &= ~MC_2;   
  
  // Toggle both LEDs together for 1 Timer Sync Mode
  TI_LMT01_XFER_LED_PxOUT ^= TI_LMT01_XFER_LED_PIN;            // toggle XFER_LED    
  TI_LMT01_TEST_LED_PxOUT ^= TI_LMT01_TEST_LED_PIN;            // toggle TEST_LED
  
  // save the result into output buffer
  pBuf[8] = pcount;
  pBuf[9] = pcount>>8;

  single_shot_state = 0;
  // remove power
  lmt01_remove_power();
  
  // stop pulse count timer
  TA0CTL &= ~MC_2;    
}
//******************************************************************************
void lmt01_oneshot_1timer_async_transistor_mode(uint8_t *pBuf)
{
  uint16_t i;
  uint16_t pcount = 0; 
  static uint8_t blink1=0;
  
  single_shot_state = 1;  
  
  // Init 1GPIO for pulse count 
  init_1gpio_mode_input();
  // enable port interrupt  
  TI_LMT01_GPIO_PULSE_COUNT_TRAN_PxIE |= TI_LMT01_GPIO_PULSE_COUNT_TRAN_PIN; 
  
  sync_to_pulse_train();

  // disable port interrupt
  TI_LMT01_GPIO_PULSE_COUNT_TRAN_PxIE &= ~TI_LMT01_GPIO_PULSE_COUNT_TRAN_PIN;
  
  // Init 1timer input for pulse count                            
  init_1timer_mode_input();
  
  // clear & restart (previously stoppped) pulse count timer
  TA0CTL |= TACLR;
  TA0CTL |= MC_2;
  
  // wait 140ms: 
  // 50ms new conversion + 50ms new pulse train + any remaining time of prev conversion
  // temp change is gradual: if prev conversion train is only 5ms (therefore remaining
  // time of prev conversion is close to 45ms), then as new conversion train would also
  // be close to 5ms, pulse count reading would happen after end of new pulse train
  for(i=0; i<=139; i++)
     __delay_cycles(4000);  // 1ms at 4MHz
  
  // stop pulse count timer
  TA0CTL &= ~MC_2;
  // save pulse count and clear timer counter 
  pcount = TA0R;
  TA0CTL |= TACLR;           

  // save the result into output buffer
  pBuf[8] = pcount;
  pBuf[9] = pcount>>8;  
  
  // Blink each LED 1 time for 1 Timer Async Mode
  if (blink1 < 2)
  {
     TI_LMT01_XFER_LED_PxOUT ^= TI_LMT01_XFER_LED_PIN;                // Set XFER_LED
     blink1++;
  } else if (blink1 < 4)
  {
     TI_LMT01_TEST_LED_PxOUT ^= TI_LMT01_TEST_LED_PIN;                // Set TEST_LED
     blink1++;
  } else
     blink1 = 0;                         
   
  single_shot_state = 0;  
}
//******************************************************************************
void init_1timer_mode_input(void)
{
   switch (lmt01_measure_mode) 
   {
     case SINGLE_GPIO_SYNC_TRANSISTOR_MODE:
     case SINGLE_GPIO_ASYNC_TRANSISTOR_MODE:
     case SINGLE_TIMER_SYNC_TRANSISTOR_MODE:
     case SINGLE_TIMER_ASYNC_TRANSISTOR_MODE:
     case TWO_TIMER_SYNC_TRANSISTOR_MODE:
     case TWO_TIMER_ASYNC_TRANSISTOR_MODE:
       {  
         // Init pulse count special function                            
         TI_LMT01_GPIO_PULSE_COUNT_TRAN_PxDIR &= ~TI_LMT01_GPIO_PULSE_COUNT_TRAN_PIN;
         TI_LMT01_GPIO_PULSE_COUNT_TRAN_PxSEL |= TI_LMT01_GPIO_PULSE_COUNT_TRAN_PIN; 
         break;
       }
     case SINGLE_GPIO_SYNC_COMPARATOR_MODE:
     case SINGLE_GPIO_ASYNC_COMPARATOR_MODE:
     case SINGLE_TIMER_SYNC_COMPARATOR_MODE:
     case SINGLE_TIMER_ASYNC_COMPARATOR_MODE:       
     case TWO_TIMER_SYNC_COMPARATOR_MODE:  
     case TWO_TIMER_ASYNC_COMPARATOR_MODE:
       {
         // Init pulse count special function                            
         TI_LMT01_GPIO_PULSE_COUNT_COMP_PxDIR &= ~TI_LMT01_GPIO_PULSE_COUNT_COMP_PIN;
         TI_LMT01_GPIO_PULSE_COUNT_COMP_PxSEL |= TI_LMT01_GPIO_PULSE_COUNT_COMP_PIN; 
         break;
       }         
   }
}
//******************************************************************************